/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "State.h"
#include "Util.h"
#include <QString>
#include <QtAlgorithms>
#include <cstring>

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register::i387Register() {
	std::memset(raw, 0, sizeof(raw));
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register::i387Register(const i387Register &other) {
	std::memcpy(raw, other.raw, sizeof(raw));
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register &State::i387Register::operator=(const State::i387Register &other) {
	if(this != &other) {
		std::memcpy(raw, other.raw, sizeof(raw));
	}
	
	return *this;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
double State::i387Register::toDouble() const {
	return static_cast<double>(raw_ldouble);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
bool State::i387Register::operator==(const State::i387Register &other) const {
	return std::memcmp(raw, other.raw, sizeof(raw)) == 0;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
bool State::i387Register::operator!=(const State::i387Register &other) const {
	return !(*this == other);
}

//------------------------------------------------------------------------------
// Name: State()
// Desc: constructor
//------------------------------------------------------------------------------
State::State() {
	clear();
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::frombackup(const State &backup) {
	ebx = backup.ebx;
	ecx = backup.ecx;
	edx = backup.edx;
	esi = backup.esi;
	edi = backup.edi;
	ebp = backup.ebp;
	eax = backup.eax;
	esp = backup.esp;
	cs_ = backup.cs_;
	ds_ = backup.ds_;
	es_ = backup.es_;
	fs_ = backup.fs_;
	gs_ = backup.gs_;
	ss_ = backup.ss_;
	eip = backup.eip;
	eflags = backup.eflags;
	orig_eax = backup.orig_eax;
	gs_base = backup.gs_base;
	fs_base = backup.fs_base;
	
	qCopy(backup.st, backup.st + 8, st);
	qCopy(backup.dr, backup.dr + 8, dr);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::toBackup(State &backup) const {
	backup.ebx = ebx;
	backup.ecx = ecx;
	backup.edx = edx;
	backup.esi = esi;
	backup.edi = edi;
	backup.ebp = ebp;
	backup.eax = eax;
	backup.esp = esp;
	backup.cs_ = cs_;
	backup.ds_ = ds_;
	backup.es_ = es_;
	backup.fs_ = fs_;
	backup.gs_ = gs_;
	backup.ss_ = ss_;
	backup.eip = eip;
	backup.eflags = eflags;
	backup.orig_eax = orig_eax;
	backup.gs_base = gs_base;
	backup.fs_base = fs_base;
	
	qCopy(st, st + 8, backup.st);
	qCopy(dr, dr + 8, backup.dr);
}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: 
//------------------------------------------------------------------------------
void State::clear() {
	eax = 0;
	ebx = 0;
	ecx = 0;
	edx = 0;
	esi = 0;
	edi = 0;
	ebp = 0;
	esp = 0;
	cs_ = 0;
	ds_ = 0;
	es_ = 0;
	fs_ = 0;
	gs_ = 0;
	ss_ = 0;
	eip = 0;
	eflags = 0;
	orig_eax = 0;
	
	std::memset(st, 0, sizeof(st));
	std::memset(dr, 0, sizeof(dr));
	
	gs_base = 0;
	fs_base = 0;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::instructionPointer() const {
	return eip;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::stackPointer() const {
	return esp;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::framePointer() const {
	return ebp;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::flags() const {
	return eflags;
}

//------------------------------------------------------------------------------
// Name: value(const QString &reg)
// Desc: a function to return the value of a register based on it's name
//------------------------------------------------------------------------------
Register State::value(const QString &reg) const {

	const QString lreg = reg.toLower();
	
	
	if(lreg == "eax")			return Register("eax", eax, Register::TYPE_GPR);
	else if(lreg == "ebx")		return Register("ebx", ebx, Register::TYPE_GPR);
	else if(lreg == "ecx")		return Register("ecx", ecx, Register::TYPE_GPR);
	else if(lreg == "edx")		return Register("edx", edx, Register::TYPE_GPR);
	else if(lreg == "ebp")		return Register("ebp", ebp, Register::TYPE_GPR);
	else if(lreg == "esp")		return Register("esp", esp, Register::TYPE_GPR);
	else if(lreg == "esi")		return Register("esi", esi, Register::TYPE_GPR);
	else if(lreg == "edi")		return Register("edi", edi, Register::TYPE_GPR);
	else if(lreg == "eip")		return Register("eip", eip, Register::TYPE_IP);
	else if(lreg == "ax")		return Register("ax", eax & 0xffff, Register::TYPE_GPR);
	else if(lreg == "bx")		return Register("bx", ebx & 0xffff, Register::TYPE_GPR);
	else if(lreg == "cx")		return Register("cx", ecx & 0xffff, Register::TYPE_GPR);
	else if(lreg == "dx")		return Register("dx", edx & 0xffff, Register::TYPE_GPR);
	else if(lreg == "bp")		return Register("bp", ebp & 0xffff, Register::TYPE_GPR);
	else if(lreg == "sp")		return Register("sp", esp & 0xffff, Register::TYPE_GPR);
	else if(lreg == "si")		return Register("si", esi & 0xffff, Register::TYPE_GPR);
	else if(lreg == "di")		return Register("di", edi & 0xffff, Register::TYPE_GPR);
	else if(lreg == "al")		return Register("al", eax & 0xff, Register::TYPE_GPR);
	else if(lreg == "bl")		return Register("bl", ebx & 0xff, Register::TYPE_GPR);
	else if(lreg == "cl")		return Register("cl", ecx & 0xff, Register::TYPE_GPR);
	else if(lreg == "dl")		return Register("dl", edx & 0xff, Register::TYPE_GPR);
	else if(lreg == "ah")		return Register("ah", (eax >> 8) & 0xff, Register::TYPE_GPR);
	else if(lreg == "bh")		return Register("bh", (ebx >> 8) & 0xff, Register::TYPE_GPR);
	else if(lreg == "ch")		return Register("ch", (ecx >> 8) & 0xff, Register::TYPE_GPR);
	else if(lreg == "dh")		return Register("dh", (edx >> 8) & 0xff, Register::TYPE_GPR);
	else if(lreg == "cs")		return Register("cs", cs, Register::TYPE_SEG, 0);
	else if(lreg == "ds")		return Register("ds", ds, Register::TYPE_SEG, 0);
	else if(lreg == "es")		return Register("es", es, Register::TYPE_SEG, 0);
	else if(lreg == "fs")		return Register("fs", fs, Register::TYPE_SEG, fs_base);
	else if(lreg == "gs")		return Register("gs", gs, Register::TYPE_SEG, gs_base);
	else if(lreg == "ss") 		return Register("ss", ss, Register::TYPE_SEG, 0);
	else if(lreg == "eflags") 	return Register("eflags", eflags, Register::TYPE_COND);
	
	return Register();
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
Register State::operator[](const QString &reg) const {
	return value(reg);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::setRegister(const QString &name, edb::reg_t value) {
	const QString lreg = name.toLower();

	if(lreg == "eax") { eax = value; }
	else if(lreg == "ebx") { ebx = value; }
	else if(lreg == "ecx") { ecx = value; }
	else if(lreg == "edx") { edx = value; }
	else if(lreg == "ebp") { ebp = value; }
	else if(lreg == "esp") { esp = value; }
	else if(lreg == "esi") { esi = value; }
	else if(lreg == "edi") { edi = value; }
	else if(lreg == "eip") { eip = value; }
	else if(lreg == "cs") { cs = value; }
	else if(lreg == "ds") { ds = value; }
	else if(lreg == "es") { es = value; }
	else if(lreg == "fs") { fs = value; }
	else if(lreg == "gs") { gs = value; }
	else if(lreg == "ss") { ss = value; }
	else if(lreg == "eflags") { eflags = value; }
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::setInstructionPointer(edb::address_t value) {
	eip = value;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::adjustStack(int bytes) {
	esp += bytes;
}

